package qses.apache.commons;

import static org.quicktheories.QuickTheory.qt;
import static org.quicktheories.generators.Generate.*;

import org.junit.Test;

public class InetAddressValidatorTestUsingQuickTheories {
  
  @Test 
  public void testIPV4Validation() {
    // Similar to what the ABNF Fuzzer example does ...
    InetAddressValidator v = InetAddressValidator.getInstance();
    qt()
    .forAll(range(0,255), range(0,255), range(0,255), range(0,255))
    .as((a,b,c,d) -> String.format("%d.%d.%d.%d", a, b, c, d))
    .check( addr -> v.isValidInet4Address(addr));
  }  
  
  @Test 
  public void testIPV6Validation() {
    // Not as thorough as ABNF Fuzzer (ipv6String yields fewer variations)s
    InetAddressValidator v = InetAddressValidator.getInstance();
    qt()
    .forAll(byteArrays(constant(16),bytes(Byte.MIN_VALUE, Byte.MAX_VALUE, (byte) 0)))
    .as(addr -> ipv6String(addr))
    .check( addr -> v.isValidInet6Address(addr));
  }
  
  private String ipv6String(byte[] data) {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < data.length; i++) {
      if (i % 2 == 1 || data[i] != 0) {
        sb.append(String.format("%02x", data[i]));
      }
      if (i % 2 == 1 && i != 15) {
        sb.append(':');
      }
    }
    return sb.toString();
  }
  
}