package qses.pwcheck;

import static org.quicktheories.QuickTheory.qt;
import static org.quicktheories.generators.Generate.byteArrays;
import static org.quicktheories.generators.Generate.bytes;
import static org.quicktheories.generators.SourceDSL.integers;
import static org.quicktheories.generators.SourceDSL.strings;

import org.junit.Test;
import org.quicktheories.core.Gen;


public class QTPasswordTest {

  static PasswordChecker CHECKER = CorrectChecker.INSTANCE;

  @Test 
  public void testStringTooSmall() {
	  qt()
	    .withFixedSeed(0)
	    .forAll(strings().basicLatinAlphabet().ofLengthBetween(0, 9))
	    .check((s) -> ! CHECKER.isPasswordOK(s));
  }
  
  @Test 
  public void testStringTooBig() {
	  qt()
	    .withFixedSeed(0)
	    .forAll(strings().basicLatinAlphabet().ofLengthBetween(21, 30))
	    .check((s) -> ! CHECKER.isPasswordOK(s));
  }
  
  @Test 
  public void testValidPasswordNoPunct() {
	  Gen<Byte> lo = bytes( (byte)'a',  (byte)'z', (byte)'a');
	  Gen<Byte> up = bytes( (byte)'A',  (byte) 'Z', (byte)'A');
	  Gen<Byte> digit = bytes( (byte) '0', (byte) '9', (byte)'0');
      Gen<Byte> combined = lo.mix(up,50).mix(digit,25); 
      Gen<byte[]> arrGen = byteArrays(integers().between(10, 20), combined);
      Gen<String> strGen = arrGen.map(ba -> new String(ba));
	  qt()
	    .withFixedSeed(0)
	    .forAll(strGen)
	    .assuming(s -> s.chars().anyMatch(Character::isLowerCase))
	    .assuming(s -> s.chars().anyMatch(Character::isUpperCase))
	    .assuming(s -> s.chars().anyMatch(Character::isDigit))
	    .check(CHECKER::isPasswordOK);
  }
 
}