package qses.tea;

import java.util.Arrays;

import org.junit.Test;

import static org.junit.Assert.*;
import static org.quicktheories.QuickTheory.qt;
import static org.quicktheories.generators.SourceDSL.*;
import static org.quicktheories.generators.Generate.*;
import org.quicktheories.core.Gen;

public class TEATest {

  @Test 
  public void testTEAWithFixedKey() {
    TEA obj = new TEA("0123456789ABCDEF".getBytes());
    qt()
    .forAll(byteArrays(integers().between(16,256), 
            bytes(Byte.MIN_VALUE, Byte.MAX_VALUE, (byte) 0)))
    .describedAs(data -> Arrays.toString(data))
    .check( data -> Arrays.equals(data, obj.decrypt(obj.encrypt(data))));
  }

  
  @Test
  public void testForAnyKey() {   
    Gen<Byte> anyValue = bytes(Byte.MIN_VALUE, Byte.MAX_VALUE, (byte) 0);
    Gen<byte[]> keyGen = byteArrays(constant(16), anyValue).describedAs(Arrays::toString);
    Gen<byte[]> dataGen = byteArrays(integers().between(1, 256), anyValue).describedAs(Arrays::toString);
    
    qt()
    .forAll(keyGen,dataGen)
    .check( (key,data) ->  {
      TEA tea = new TEA(key);
      return Arrays.equals( tea.decrypt(tea.encrypt(data)), data);
    });
  }
  
  @Test
  public void testSimplistic() {   
    byte[] data = new byte[16];
    TEA tea = new TEA(new byte[16]);
    
    assertTrue( Arrays.equals(data, tea.decrypt(tea.encrypt(data))) );
  }
 
}